/*
 * AvrStamp
 * A simple plugable and breadboardable board
 *
 * Copyright (C) 2012  B.Stultiens
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdint.h>
#include <string.h>

#include <clock.h>

#define TIMER1CTC	(2000-1)		/* CTC match value for 1000Hz @1:8 prescale*/


#define clock_countdown(x)	do { if(clock_timers[x].state == TIMER_RUNNING) { \
					if(!--clock_timers[x].cnt) \
						clock_timers[x].state = TIMER_EXPIRED; \
					} \
				} while(0)

volatile clock_timer_t clock_timers[MAXTIMERS];
volatile uint32_t clock_secs;			/* Seconds (unix epoch if we base it) */
volatile uint16_t clock_msecs;			/* Millisecond resolution [0..999] */

void clock_init(void)
{
	clock_secs = 0;
	clock_msecs = 0;
	memset((void *)&clock_timers[0], 0, sizeof(clock_timers));
	TCCR1A = 0;				/* CTC mode */
	TCCR1B = _BV(WGM12) | _BV(CS11);	/* timer2 prescaler 1:8 */
	OCR1A = TIMER1CTC;
	TIMSK = _BV(OCIE1A);			/* Interrupt on compare A match */
	TCNT1 = 0;				/* Clear counter */
}


ISR(TIMER1_COMPA_vect)
{
	if(++clock_msecs == 1000) {
		clock_msecs = 0;
		clock_secs++;
	}

#if 0
	for(uint8_t i = 0; i < MAXTIMERS; i++) {
		if(clock_timers[i].state == TIMER_RUNNING) {
			if(!--clock_timers[i].cnt)
				clock_timers[i].state = TIMER_EXPIRED;
		}
	}
#else
	/* Loop-unroll is faster */
	/* XXX: But beware, changing the list must be reflected here */
	clock_countdown(TIMER_BLINKY);
	clock_countdown(TIMER_WHEEL);
#endif
}
